import 'package:flutter/cupertino.dart';
import 'package:taastrap/taastrap.dart';
import 'package:get_it/get_it.dart';

import 'element.dart';

/// It creates a responsive rows
class ResponsiveRow extends StatelessWidget{

  /// The children to make responsive with a mapping sizes
  final Map<Widget, Map<String, int>> children;

  /// The list controller
  final ScrollController? controller;

  /// The axis direction of the list scroll
  final Axis axis;

  /// Boolean flag to check if the child is custom or a default child
  final bool customChild;

  /// Padding for internal custom child
  final EdgeInsets? customInternalPadding;

  ResponsiveRow({Key? key,
    required this.children,
    this.axis = Axis.vertical,
    this.controller,
    this.customChild = false,
    this.customInternalPadding
  }): super(key: key);

  /// It retrieves the children in many row as it needs
  List<Row> _getChildren(int deviceType, BuildContext context){
    List<Row> finals = [];
    List<Widget> tmpChildren = [];
    double counter = 0;
    this.children.forEach((Widget key, Map<String, int> value) {
      if(this.customChild){
        ResponsiveWidget tmpResponsiveWidget = key as ResponsiveWidget;
        Widget tmpWidget = Flexible(
          flex: 0,
          fit: FlexFit.tight,
          child: Padding(
            padding: this.customInternalPadding != null ? EdgeInsets.zero : this.customInternalPadding!,
            child: tmpResponsiveWidget,
          ),
        );
        double maxWidth = context.size!.width;
        double leftWidget = this.customInternalPadding != null ? this.customInternalPadding!.left : 0;
        double rightWidget = this.customInternalPadding != null ? this.customInternalPadding!.right : 0;
        double currentSize = tmpResponsiveWidget.getSize() + leftWidget + rightWidget;
        double tmpCounter = counter + currentSize;
        if(tmpCounter >= maxWidth){
          finals.add(
            Row(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: List<Widget>.generate(tmpChildren.length, ((index) => tmpChildren[index])),
            )
          );
          tmpChildren = [tmpWidget];
          counter = currentSize;
        }
        else{
          tmpChildren.add(tmpWidget);
          counter += currentSize;
        }
      }
      else{
        int? xs = value['xs'] ?? null;
        int? sm = value['sm'] ?? null;
        int? md = value['md'] ?? null;
        int? lg = value['lg'] ?? null;
        int? xl = value['xl'] ?? null;
        ResponsiveElement tmp = ResponsiveElement(
            element: key,
            deviceType: deviceType,
            xs: xs,
            sm: sm,
            md: md,
            lg: lg,
            xl: xl
        );
        double currentSize = tmp.getSizeElement().toDouble();
        double tmpCounter = counter + currentSize;
        if(tmpCounter > 12){
          finals.add(
              Row(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: List<Widget>.generate(tmpChildren.length, ((index) => tmpChildren[index])),
              )
          );
          tmpChildren = [tmp];
          counter = currentSize;
        }
        else{
          tmpChildren.add(tmp);
          counter += currentSize;
        }
      }
    });
    finals.add(Row(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: List<Widget>.generate(tmpChildren.length, (index) => tmpChildren[index]),
    ));
    return finals;
  }



  @override
  Widget build(BuildContext context) {

    int _deviceType = GetIt.I<Generic>().deviceType();

    return LayoutBuilder(builder: (contextBuilder, constraints){
      return this.controller == null ? (this.axis == Axis.vertical ? Column(
        children: this._getChildren(_deviceType, contextBuilder),
      ) : Row(
        children: this._getChildren(_deviceType, contextBuilder),
      )) : ListView(
        controller: this.controller,
        scrollDirection: this.axis,
        children: this._getChildren(_deviceType, contextBuilder),
      );
    });
  }

}